# Written by Yaniv Erlich (erlich@cshl.edu)
# calculates the expected number of flase positives for random k-set pooling design.
# input parameters: number of specimesn, lower number of tests, upper number of tests, lower weight, upper weight, probability of mutant
# the function will iterate over each test, weight combination and calculates the expected number of false positives.
# output parameters: array with the number of false positives, ninimum false positives, maximum false positives.

# for more details see Bruno et al. 1995 - "Efficient Pooling Designs for Library Screening"
# if you feel there is a bug in the script please send an email to erlich@cshl.edu.
# if you use the script please cite also our genome research paper (2009).

# you may modify, copy or do whatever you want.

use strict;
use warnings;
use Hypergeom qw(hypergeom gammln);
our @ISA = ("Exporter");
our @EXPORT = qw(scan_parameters);


sub test {
    print expected_false_positive(1000,5,30,10/1000),"\n";
    #should print   390.1053
}

sub scan_parameters {
    # the function that exported out
    my ($specimens,$test_start,$test_stop,$w_start,$w_stop,$prob) = @_;

    my @r;
    my $min = 1000000000;
    my $max = 0;


    foreach my $w ($w_start..$w_stop) {
        foreach my $test ($test_start..$test_stop) {
            my $s = expected_false_positive($specimens,$w,$test,$prob);
            $r[$w][$test] = $s;
            if ($min > $s) {$min = $s};
            if ($max < $s) {$max = $s};
        }
    }

    return (\@r,$min,$max);
}

sub expected_false_positive {
    #the main function
    #notice that the notation of the parameters changed from Bruno et al.

    my ($specimens,$w,$tests,$prob) = @_;

    my $grand_sum = 0;

    foreach my $i (0..$specimens) {

        my $first_term = $specimens - $i;
        my $second_term = binomial($i,$specimens,$prob);
        my $third_term = 0;


        foreach my $jj (0..$w) {

            my $c_k_jj = nchoosek($w,$jj);
            my $plus_minus = (-1) ** $jj;
            my $z_hyper_i = hypergeom(0,$tests,$jj,$w) ** $i;

            my $prod = $c_k_jj * $plus_minus * $z_hyper_i;
            $third_term += $prod;
        }


        my $grand_prod = $first_term * $second_term * $third_term;
        $grand_sum += $grand_prod;
    }

    return $grand_sum; #number of false positives
}


sub binomial {
    #calculates the binomial pdf.
    #k - number of hits
    #n - tests
    #p - probability of a hit

    my ($k,$n,$p) = @_;
    my $first  = $k      * log ($p);
    my $second = ($n-$k) * log (1-$p);
    my $third  = nchoosek_ln($n,$k);
    my $r = $first + $second + $third;

    return exp($r); 
}




sub nchoosek_ln {
    #calculates log of n!/(k!*(n-k)!)
    my ($n,$k) = @_;

    my $n_ln = gammln($n + 1.0);
    my $k_ln = gammln($k + 1.0);
    my $n_k_ln = gammln(($n-$k)+1.0);

    my $r = $n_ln - $n_k_ln - $k_ln;

    return $r;
}



sub nchoosek {
    #calculates n!/(k!*(n-k)!)
    my ($n,$k) = @_;

    my $n_ln = gammln($n + 1.0);
    my $k_ln = gammln($k + 1.0);
    my $n_k_ln = gammln(($n-$k)+1.0);

    my $r = $n_ln - $n_k_ln - $k_ln;
    return exp($r);

}